import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

public class Main {
    static final String[] STRINGS = {
            "   refid \"HUDMenu/instance/init\"",
            "  trait slot QName(PackageNamespace(\"\"), \"BGSCodeObj\") type QName(PackageNamespace(\"\"), \"Object\") end",
            "    maxstack",
            "     pushscope",
            "     pop",
            "    end ; code"
    };
    static final String[] STRINGS2 = {
            " #include \"Shared/AS3/VaultBoyImageLoader.instance/SWFLoad/inline_method.method.asasm\""
    };

    public static void main(String[] args) {

        int numPatchSuccess = 0;

        try {
            List<String> patchContent = new ArrayList<>(Files.readAllLines(Paths.get("hudframework\\HUDMenu-0\\HUDMenu.class.asasm"), StandardCharsets.UTF_8));
            List<String> fileContent = new ArrayList<>(Files.readAllLines(Paths.get("HUDMenu_patched-0\\HUDMenu.class.asasm"), StandardCharsets.UTF_8));

            //System.out.println(fileContent);

            // PHASE 1: Add NEED_ACTIVATION flag.
            int idx0 = fileContent.indexOf(STRINGS[0]);
            int p_idx0 = patchContent.indexOf(STRINGS[0]);
            if (idx0 > -1) {
                if (!fileContent.get(idx0+1).equals(patchContent.get(p_idx0+1))) {
                    fileContent.add(idx0+1, patchContent.get(p_idx0+1));
                }
                numPatchSuccess++;
            }

            // PHASE 2: Add HUDFrameworkReady and commandBuffer variables.
            int idx1 = fileContent.indexOf(STRINGS[1]);
            int p_idx1 = patchContent.indexOf(STRINGS[1]);
            if (idx1 > -1) {
                if (!fileContent.get(idx1+1).equals(patchContent.get(p_idx1+1))) {
                    fileContent.add(idx1+1, patchContent.get(p_idx1+1));
                    fileContent.add(idx1+2, patchContent.get(p_idx1+2));
                }
                numPatchSuccess++;
            }

            // PHASE 3: Prepare to modify constructor.
            // Compare maxstack and pick higher.

            // First get maxstack of patch files
            int patchLineIndex = -1;
            int maxstack = 0, localcount = 0, initscopedepth = 0, maxscopedepth = 0;

            for (ListIterator<String> iter = patchContent.listIterator(); iter.hasNext();) {
                int lineIndex = iter.nextIndex();
                String line = iter.next();
                if (line.contains(STRINGS[2])) {
                    patchLineIndex = lineIndex;
                    maxstack        = Character.getNumericValue(line.charAt(line.length() - 1));
                    line = iter.next();
                    localcount      = Character.getNumericValue(line.charAt(line.length() - 1));
                    line = iter.next();
                    initscopedepth  = Character.getNumericValue(line.charAt(line.length() - 1));
                    line = iter.next();
                    maxscopedepth   = Character.getNumericValue(line.charAt(line.length() - 1));
                    break;
                }
            }

            // Now get maxstack of vanilla files and compare.
            // Replace vanilla values with ours if ours is bigger.
            for (ListIterator<String> iter = fileContent.listIterator(); iter.hasNext();) {
                String line = iter.next();
                if (line.contains(STRINGS[2])) {
                    if (maxstack > Character.getNumericValue(line.charAt(line.length() - 1))) {
                        iter.set(patchContent.get(patchLineIndex));
                    }
                    line = iter.next();
                    if (localcount > Character.getNumericValue(line.charAt(line.length() - 1))) {
                        iter.set(patchContent.get(patchLineIndex+1));
                    }
                    line = iter.next();
                    if (initscopedepth > Character.getNumericValue(line.charAt(line.length() - 1))) {
                        iter.set(patchContent.get(patchLineIndex+2));
                    }
                    line = iter.next();
                    if (maxscopedepth > Character.getNumericValue(line.charAt(line.length() - 1))) {
                        iter.set(patchContent.get(patchLineIndex+3));
                    }
                    numPatchSuccess++;
                    break;
                }
            }

            // PHASE 4: Modify constructor body

            // Look for starting section in patchContent
            int idx3 = fileContent.indexOf(STRINGS[3]);
            int p_idx3 = patchContent.indexOf(STRINGS[3]);

            // Look for ending section in patchContent
            int p_idx4 = patchContent.indexOf(STRINGS[4]);

            if (idx3 > -1) {
                // Check two lines down (ignore the blank line) to look for newactivation.
                if (!fileContent.get(idx3 + 2).equals(patchContent.get(p_idx3 + 2))) {
                    // Take the first line after the starting match string to one line after the last string (the newline)
                    fileContent.addAll(idx3+1, patchContent.subList(p_idx3 + 1, p_idx4 + 1));
                }
                numPatchSuccess++;
            }

            // PHASE 5: Finally, add loadHUDFramework trait slot.
            int idx5 = fileContent.indexOf(STRINGS[5]);
            int p_idx5 = patchContent.indexOf(STRINGS[5]);
            if (idx5 > -1) {
                if (!fileContent.get(idx5+1).equals(patchContent.get(p_idx5+1))) {
                    fileContent.add(idx5+1, patchContent.get(p_idx5+1));
                }
                numPatchSuccess++;
            }

            //System.out.println(patchContent);
            Files.write(Paths.get("HUDMenu_patched-0\\HUDMenu.class.asasm"), fileContent, StandardCharsets.UTF_8);


            // Part 2:
            patchContent = new ArrayList<>(Files.readAllLines(Paths.get("hudframework\\HUDMenu-0\\HUDMenu-0.main.asasm"), StandardCharsets.UTF_8));
            fileContent = new ArrayList<>(Files.readAllLines(Paths.get("HUDMenu_patched-0\\HUDMenu_patched-0.main.asasm"), StandardCharsets.UTF_8));

            // Look for starting section for both files
            idx0 = fileContent.indexOf(STRINGS2[0]);
            p_idx0 = patchContent.indexOf(STRINGS2[0]);

            int end_line_idx = p_idx0;

            while (patchContent.get(end_line_idx).length() > 0) {
                end_line_idx++;
            }

            if (idx0 > -1) {
                if (!fileContent.get(idx0+2).equals(patchContent.get(p_idx0+2))) {
                    fileContent.addAll(idx0+1, patchContent.subList(p_idx0 + 1, end_line_idx));
                }
                numPatchSuccess++;
            }

            Files.write(Paths.get("HUDMenu_patched-0\\HUDMenu_patched-0.main.asasm"), fileContent, StandardCharsets.UTF_8);

            System.out.println("Successfully patched " + numPatchSuccess + "/6 code blocks.");

        } catch (IOException e) {
            System.out.println("IOException.");
        }
    }
}
